@@ -1,6 +1,6 @@
 ;;; lang/javascript/mach.scm
 ;;;
-;;; Copyright (C) 2015,2017 Matthew R. Wette
+;;; Copyright (C) 2015-2017 Matthew R. Wette
 ;;;
 ;;; This program is free software: you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by 
@@ -43,7 +43,12 @@
 
 (define js-spec
   (lalr-spec
-   (notice (string-append "Copyright 2016 Matthew R. Wette" lang-crn-lic))
+   (notice (string-append "Copyright 2015-2017 Matthew R. Wette" lang-crn-lic))
+   (reserve "abstract" "boolean" "byte" "char" "class" "const" "debugger"
+	    "double" "enum" "export" "extends" "final" "float" "goto"
+	    "implemements" "import" "int" "interface" "long" "native" 
+	    "package" "private" "protected" "public" "short" "static"
+	    "super" "synchronized" "throws" "transient" "volatile")
    (prec< "then" "else")
    (start Program)
    (grammar
@@ -70,8 +75,7 @@
      (Identifier ($$ `(PrimaryExpression ,$1)))
      (Literal ($$ `(PrimaryExpression ,$1)))
      (ArrayLiteral ($$ `(PrimaryExpression ,$1)))
-     ;; until we get $with-prune working:
-     #;(ObjectLiteral ($$ `(PrimaryExpression ,$1)))
+     ;;(ObjectLiteral ($$ `(PrimaryExpression ,$1)))
      ("(" Expression ")" ($$ $2))
      )
 
@@ -117,8 +121,7 @@
 
     (MemberExpression
      (PrimaryExpression)
-     ;; Until we get $with-prune working:
-     #;(FunctionExpression)
+     ;;(FunctionExpression)
      (MemberExpression "[" Expression "]" ($$ `(ary-ref ,$3 ,$1)))
      (MemberExpression "." Identifier ($$ `(obj-ref ,$3 ,$1)))
      ("new" MemberExpression Arguments ($$ `(new ,$2 ,$3)))
@@ -210,13 +213,22 @@
 			   ($$ `(ge ,$1 ,$3)))
      (RelationalExpression "instanceof" ShiftExpression
 			   ($$ `(instanceof ,$1 ,$3)))
-     ;; until we get $with-prune working:
-     #;(RelationalInExpression)
-     )
-    #;(RelationalInExpression
      (RelationalExpression "in" ShiftExpression
 			   ($$ `(in ,$1 ,$3)))
      )
+    (RelationalExpressionNoIn
+     (ShiftExpression)
+     (RelationalExpressionNoIn "<" ShiftExpression
+			       ($$ `(lt ,$1 ,$3)))
+     (RelationalExpressionNoIn ">" ShiftExpression
+			       ($$ `(gt ,$1 ,$3)))
+     (RelationalExpressionNoIn "<=" ShiftExpression
+			       ($$ `(le ,$1 ,$3)))
+     (RelationalExpressionNoIn ">=" ShiftExpression
+			       ($$ `(ge ,$1 ,$3)))
+     (RelationalExpressionNoIn "instanceof" ShiftExpression
+			       ($$ `(instanceof ,$1 ,$3)))
+     )
     
     (EqualityExpression
      (RelationalExpression)
@@ -229,48 +241,95 @@
      (EqualityExpression "!==" RelationalExpression
 			 ($$ `(neq-eq ,$1 ,$3)))
      )
+    (EqualityExpressionNoIn
+     (RelationalExpressionNoIn)
+     (EqualityExpressionNoIn "==" RelationalExpressionNoIn
+			     ($$ `(eq ,$1 ,$3)))
+     (EqualityExpressionNoIn "!=" RelationalExpressionNoIn
+			     ($$ `(neq ,$1 ,$3)))
+     (EqualityExpressionNoIn "===" RelationalExpressionNoIn
+			     ($$ `(eq-eq ,$1 ,$3)))
+     (EqualityExpressionNoIn "!==" RelationalExpressionNoIn
+			     ($$ `(neq-eq ,$1 ,$3)))
+     )
 
     (BitwiseANDExpression
      (EqualityExpression)
      (BitwiseANDExpression "&" EqualityExpression
 			   ($$ `(bit-and ,$1 ,$3)))
      )
+    (BitwiseANDExpressionNoIn
+     (EqualityExpressionNoIn)
+     (BitwiseANDExpressionNoIn "&" EqualityExpressionNoIn
+			   ($$ `(bit-and ,$1 ,$3)))
+     )
 
     (BitwiseXORExpression
      (BitwiseANDExpression)
      (BitwiseXORExpression "^" BitwiseANDExpression
 			   ($$ `(bit-xor ,$1 ,$3)))
      )
+    (BitwiseXORExpressionNoIn
+     (BitwiseANDExpressionNoIn)
+     (BitwiseXORExpressionNoIn "^" BitwiseANDExpressionNoIn
+			       ($$ `(bit-xor ,$1 ,$3)))
+     )
 
     (BitwiseORExpression
      (BitwiseXORExpression)
      (BitwiseORExpression "|" BitwiseXORExpression
 			  ($$ `(bit-or ,$1 ,$3)))
      )
+    (BitwiseORExpressionNoIn
+     (BitwiseXORExpressionNoIn)
+     (BitwiseORExpressionNoIn "|" BitwiseXORExpressionNoIn
+			      ($$ `(bit-or ,$1 ,$3)))
+     )
 
     (LogicalANDExpression
      (BitwiseORExpression)
      (LogicalANDExpression "&&" BitwiseORExpression
 			   ($$ `(and ,$1 ,$3)))
      )
+    (LogicalANDExpressionNoIn
+     (BitwiseORExpressionNoIn)
+     (LogicalANDExpressionNoIn "&&" BitwiseORExpressionNoIn
+			       ($$ `(and ,$1 ,$3)))
+     )
 
     (LogicalORExpression
      (LogicalANDExpression)
      (LogicalORExpression "||" LogicalANDExpression
 			  ($$ `(or ,$1 ,$3)))
      )
+    (LogicalORExpressionNoIn
+     (LogicalANDExpressionNoIn)
+     (LogicalORExpressionNoIn "||" LogicalANDExpressionNoIn
+			  ($$ `(or ,$1 ,$3)))
+     )
 
     (ConditionalExpression
      (LogicalORExpression)
      (LogicalORExpression "?" AssignmentExpression ":" AssignmentExpression
 			  ($$ `(ConditionalExpression ,$1 ,$3 ,$5)))
      )
+    (ConditionalExpressionNoIn
+     (LogicalORExpressionNoIn)
+     (LogicalORExpressionNoIn "?" AssignmentExpression
+			      ":" AssignmentExpressionNoIn
+			  ($$ `(ConditionalExpression ,$1 ,$3 ,$5)))
+     )
     
     (AssignmentExpression
      (ConditionalExpression)
      (LeftHandSideExpression AssignmentOperator AssignmentExpression
 			     ($$ `(AssignmentExpression ,$1 ,$2 ,$3)))
      )
+    (AssignmentExpressionNoIn
+     (ConditionalExpressionNoIn)
+     (LeftHandSideExpression AssignmentOperator AssignmentExpressionNoIn
+			     ($$ `(AssignmentExpression ,$1 ,$2 ,$3)))
+     )
 
     (AssignmentOperator
      ;; todo
@@ -296,8 +355,12 @@
 	      (make-tl 'expr-list $1 $3))))
      )
     (ExpressionNoIn
-     #;($with Expression ($prune RelationalInExpression))
-     (Expression)
+     (AssignmentExpressionNoIn)
+     (ExpressionNoIn
+      "," AssignmentExpressionNoIn
+      ($$ (if (and (pair? (car $1)) (eqv? 'expr-list (caar $1)))
+	      (tl-append $1 $3)
+	      (make-tl 'expr-list $1 $3))))
      )
 	    
     ;; A.4
@@ -338,9 +401,9 @@
      (VariableDeclarationList "," VariableDeclaration ($$ (tl-append $1 $3)))
      )
     (VariableDeclarationListNoIn
-     #;($with VariableDeclarationList ($prune RelationalInExpression))
-     ;; ==[Until we get $with-prune working]==>
-     (VariableDeclarationList)
+     (VariableDeclarationNoIn ($$ (make-tl 'VariableDeclarationList $1)))
+     (VariableDeclarationListNoIn "," VariableDeclarationNoIn
+				  ($$ (tl-append $1 $3)))
      )
 
     (VariableDeclaration
@@ -348,26 +411,23 @@
      (Identifier ($$ `(VariableDeclaration ,$1)))
      )
     (VariableDeclarationNoIn
-     #;($with VariableDeclaration ($prune RelationalInExpression))
-     ;; ==[Until we get $with-prune working]==>
-     (VariableDeclaration)
+     (Identifier InitializerNoIn ($$ `(VariableDeclaration ,$1 ,$2)))
+     (Identifier ($$ `(VariableDeclaration ,$1)))
      )
 
     (Initializer
      ("=" AssignmentExpression ($$ `(Initializer ,$2)))
      )
+    (InitializerNoIn
+     ("=" AssignmentExpressionNoIn ($$ `(Initializer ,$2)))
+     )
 
     (EmptyStatement
      (";" ($$ '(EmptyStatement)))
      )
 
     (ExpressionStatement
-     ;; spec says: Reject if lookahead "{" "," "function".
-     #;(($with Expression
-	     ($prune FunctionExpression)
-	     ($prune ObjectLiteral))
-     ";")
-     ;; ==[until we get $with-prune working]==>
+     ;; spec says: Reject if lookahead "{" | "function".
      (Expression ";" ($$ `(ExpressionStatement ,$1)))
      )
 
@@ -384,12 +444,18 @@
      ("while" "(" Expression ")" Statement
       ($$ `(while ,$3 ,$5)))
      ("for" "(" OptExprStmtNoIn OptExprStmt OptExprClose Statement
-      ;;($$ `(for ,$3 ,$
+      ($$ `(for $3 $4 $5 $6))
       )
      ("for" "(" "var" VariableDeclarationListNoIn ";" OptExprStmt
-      OptExprClose Statement)
-     ("for" "(" LeftHandSideExpression "in" Expression ")" Statement)
-     ("for" "(" "var" VariableDeclarationNoIn "in" Expression ")" Statement)
+      OptExprClose Statement
+      ($$ `(for $4 $6 $7 $8))		; ???
+      )
+     ("for" "(" LeftHandSideExpression "in" Expression ")" Statement
+      ($$ `(for-in $3 $5 $7))		; ???
+      )
+     ("for" "(" "var" VariableDeclarationNoIn "in" Expression ")" Statement
+      ($$ `(for-in $4 $6 $8))		; ???
+      )
      )
     (OptExprStmtNoIn
      (":" ($$ `(Expression)))
